@@ -134,6 +134,7 @@ struct btd_adapter {
 
 	struct hci_dev dev;		/* hci info */
 	gboolean pairable;		/* pairable state */
+	gboolean initialized;
 
 	gboolean off_requested;		/* DEVDOWN ioctl was called */
 
@@ -2246,7 +2247,7 @@ int adapter_get_discover_type(struct btd_adapter *adapter)
 	return type;
 }
 
-void btd_adapter_get_state(struct btd_adapter *adapter, uint8_t *mode,
+void btd_adapter_get_mode(struct btd_adapter *adapter, uint8_t *mode,
 					uint8_t *on_mode, gboolean *pairable)
 {
 	char str[14], address[18];
@@ -2263,9 +2264,14 @@ void btd_adapter_get_state(struct btd_adapter *adapter, uint8_t *mode,
 	}
 
 	if (on_mode) {
-		if (main_opts.remember_powered == FALSE)
-			*on_mode = main_opts.mode;
-		else if (read_on_mode(address, str, sizeof(str)) < 0)
+		if (main_opts.remember_powered == FALSE) {
+			if (adapter->initialized)
+				*on_mode = get_mode(&adapter->bdaddr, "on");
+			else {
+				*on_mode = main_opts.mode;
+				adapter->initialized = TRUE;
+			}
+		} else if (read_on_mode(address, str, sizeof(str)) < 0)
 			*on_mode = main_opts.mode;
 		else
 			*on_mode = get_mode(&adapter->bdaddr, str);
@@ -2841,9 +2847,9 @@ void adapter_emit_device_found(struct btd_adapter *adapter,
 				"RSSI", DBUS_TYPE_INT16, &rssi,
 				"Name", DBUS_TYPE_STRING, &dev->name,
 				"Paired", DBUS_TYPE_BOOLEAN, &paired,
-				"UUIDs", DBUS_TYPE_ARRAY, &dev->uuids,
 				"Broadcaster", DBUS_TYPE_BOOLEAN, &broadcaster,
-				dev->uuid_count, NULL);
+				"UUIDs", DBUS_TYPE_ARRAY, &dev->uuids, uuid_count,
+				NULL);
 		return;
 	}
 
@@ -2867,7 +2873,7 @@ void adapter_emit_device_found(struct btd_adapter *adapter,
 			"Alias", DBUS_TYPE_STRING, &alias,
 			"LegacyPairing", DBUS_TYPE_BOOLEAN, &dev->legacy,
 			"Paired", DBUS_TYPE_BOOLEAN, &paired,
-			"UUIDs", DBUS_TYPE_ARRAY, &dev->uuids, dev->uuid_count,
+			"UUIDs", DBUS_TYPE_ARRAY, &dev->uuids, uuid_count,
 			NULL);
 
 	g_free(alias);
@@ -2917,10 +2923,18 @@ static void remove_same_uuid(gpointer data, gpointer user_data)
 	}
 }
 
+static void dev_prepend_uuid(gpointer data, gpointer user_data)
+{
+	struct remote_dev_info *dev = user_data;
+	char *new_uuid = data;
+
+	dev->services = g_slist_prepend(dev->services, g_strdup(new_uuid));
+}
+
 void adapter_update_device_from_info(struct btd_adapter *adapter,
-						bdaddr_t bdaddr, int8_t rssi,
-						uint8_t evt_type, char *name,
-						GSList *services, uint8_t flags)
+					bdaddr_t bdaddr, int8_t rssi,
+					uint8_t evt_type, const char *name,
+					GSList *services, uint8_t flags)
 {
 	struct remote_dev_info *dev;
 	gboolean new_dev;
@@ -2939,13 +2953,13 @@ void adapter_update_device_from_info(struct btd_adapter *adapter,
 						(GCompareFunc) dev_rssi_cmp);
 
 	g_slist_foreach(services, remove_same_uuid, dev);
-	dev->services = g_slist_concat(dev->services, services);
+	g_slist_foreach(services, dev_prepend_uuid, dev);
 
 	dev->flags = flags;
 
 	if (name) {
 		g_free(dev->name);
-		dev->name = name;
+		dev->name = g_strdup(name);
 	}
 
 	/* FIXME: check if other information was changed before emitting the
@@ -2983,7 +2997,7 @@ void adapter_update_found_devices(struct btd_adapter *adapter, bdaddr_t *bdaddr,
 						(GCompareFunc) dev_rssi_cmp);
 
 	g_slist_foreach(services, remove_same_uuid, dev);
-	dev->services = g_slist_concat(dev->services, services);
+	g_slist_foreach(services, dev_prepend_uuid, dev);
 
 	adapter_emit_device_found(adapter, dev);
 }
@@ -3009,6 +3023,8 @@ void adapter_mode_changed(struct btd_adapter *adapter, uint8_t scan_mode)
 	const gchar *path = adapter_get_path(adapter);
 	gboolean discoverable, pairable;
 
+	DBG("old 0x%02x new 0x%02x", adapter->scan_mode, scan_mode);
+
 	if (adapter->scan_mode == scan_mode)
 		return;
 
@@ -3372,16 +3388,12 @@ int btd_adapter_restore_powered(struct btd_adapter *adapter)
 
 int btd_adapter_switch_online(struct btd_adapter *adapter)
 {
-	gboolean discoverable;
-
 	if (!adapter_ops)
 		return -EINVAL;
 
 	if (adapter->up)
 		return 0;
 
-	discoverable = get_mode(&adapter->bdaddr, "on") == MODE_DISCOVERABLE;
-
 	return adapter_ops->set_powered(adapter->dev_id, TRUE);
 }
 
